package main

import (
	"fmt"
	"testing"
)

type NumberGenerator interface {
        generateEvenNumbers() []int
}

type EvenNumber struct{}

func (e *EvenNumber) generateEvenNumbers() []int {
        evenNumbers := make([]int, 5)
        for i := 0; i < 5; i++ {
                evenNumbers[i] = 2 * (i + 1)
        }
        return evenNumbers
}

type TestNumberGenerator struct{}

func (t *TestNumberGenerator) generateEvenNumbers() []int {
        evenNumbers := make([]int, 10)
        for i := 0; i < 5; i++ {
                evenNumbers[i] = 2 * (i + 1)
        }
        return evenNumbers
}

type BrokenNumberGenerator struct{}

func (b *BrokenNumberGenerator) generateEvenNumbers() []int {
        // This function will not generate even numbers but it won't panic either
        return make([]int, 0)
}

func main() {
        evenNumber := &EvenNumber{}
        generatedNumbers := evenNumber.generateEvenNumbers()
        fmt.Println(generatedNumbers)

        brokenNumberGenerator := &BrokenNumberGenerator{}
        generatedNumbers = brokenNumberGenerator.generateEvenNumbers()
        fmt.Println(generatedNumbers)

        testNumberGenerator := &TestNumberGenerator{}
        generatedNumbers = testNumberGenerator.generateEvenNumbers()
        fmt.Println(generatedNumbers)
}

func TestGenerateEvenNumbers(t *testing.T) {
        evenNumber := &EvenNumber{}
        generatedNumbers := evenNumber.generateEvenNumbers()

        if len(generatedNumbers) != 5 {
                t.Errorf("Expected 5 numbers but got %d", len(generatedNumbers))
        }
        for _, number := range generatedNumbers {
                if number%2 == 1 {
                        t.Errorf("Expected all numbers to be even but got an odd number: %d", number)
                }
        }

        testNumberGenerator := &TestNumberGenerator{}
        generatedNumbers = testNumberGenerator.generateEvenNumbers()
        if len(generatedNumbers) != 10 {
                t.Errorf("Expected 10 numbers but got %d", len(generatedNumbers))
        }
        for _, number := range generatedNumbers[:5] {
                if number%2 == 1 {
                        t.Errorf("Expected all first 5 numbers to be even but got an odd number: %d", number)
                }
        }

        brokenNumberGenerator := &BrokenNumberGenerator{}
        generatedNumbers = brokenNumberGenerator.generateEvenNumbers()
        if len(generatedNumbers) != 0 {
                t.Errorf("Expected zero numbers but got %d", len(generatedNumbers))
        }
}

func TestGenerateEvenNumbersSize(t *testing.T) {
        evenNumber := &EvenNumber{}
        generatedNumbers := evenNumber.generateEvenNumbers()

        if len(generatedNumbers) == 5 && generatedNumbers[4] != 10 {
                t.Errorf("Expected last number to be 10 but got %d", generatedNumbers[4])
        }
}

func TestGenerateEvenNumbersBroken(t *testing.T) {
        brokenNumberGenerator := &BrokenNumberGenerator{}
        generatedNumbers := brokenNumberGenerator.generateEvenNumbers()

        if len(generatedNumbers) != 0 {
                t.Errorf("Expected zero numbers but got %d", len(generatedNumbers))
        }
}