package main

import (
	"bufio"
	"fmt"
	displayindicator "llama/display-indicator"
	"llama/extraction"
	"llama/go_compiler"
	ollamaimplementation "llama/ollama-implementation"
	"os"
	"strings"
)

func main() {
	reader := bufio.NewReader(os.Stdin)
	var conversationContext []int // Variable to store conversation context

	fmt.Print("Enter your prompt (or type 'exit' to quit): ")
	userPrompt, _ := reader.ReadString('\n')
	userPrompt = strings.TrimSpace(userPrompt)

	for {

		if userPrompt == "exit" {
			fmt.Println("Exiting the program.")
			break
		}

		var modifiedPrompt = userPrompt + extraction.GoPrompt

		fmt.Println("Prompt received. Generating response...")

		// Start a go routine to display a waiting indicator while the response is being generated
		done := make(chan bool)
		go displayindicator.DisplayLoadingIndicator(done)

		// Generate response using Ollama API, passing the context
		response, updatedContext, err := ollamaimplementation.GetOllamaResponse(modifiedPrompt, conversationContext)

		// Signal the waiting indicator to stop
		done <- true

		if err != nil {
			fmt.Println("Error generating response:", err)
			continue
		}

		// Update the conversation context with the response
		conversationContext = updatedContext

		generatedCode, err_extract := extraction.Extract(response) // Handle error with string

		if err_extract != nil {
			fmt.Printf("The LLM gave a improper string in response: %v", response)
			userPrompt = "exit"
			continue
		}

		fmt.Println("Ollama's response:", generatedCode)

		output, err := go_compiler.NewGoCompiler().CheckCompileErrors([]byte(generatedCode))

		if err != nil {
			fmt.Printf("The code did not compile and contains the following errors: %v\n", string(output))
			userPrompt = "Following are the errors, please fix the code. Write it again, and write only source code along with same test cases with no further explanation. The format should be ```rust <yourcode + testcases> ```  :\n" +  string(output) 
		} else {
			fmt.Printf("Compiled successfully. Here is the output: %v", string(output))
			userPrompt = "exit"
		}
	}
}
